#include <TF1.h>

#include "TH2D.h"
#include "TH2I.h"

#include "TROOT.h"
#include "TRint.h"
#include "TTree.h"
#include "TList.h"
#include <TH2.h>
#include "TSelector.h"
#include <TStyle.h>
#include <TString.h>
#include "TLorentzVector.h"
#include <fstream>    // for fstream
#include <iostream>   // for cout, endl
#include <stdio.h>
#include <string>

#include <TH1.h>
#include <TRandom2.h>
#include <TRandom3.h>

#include <TSystem.h>

#include "TVector3.h"
#include <math.h>
#include <signal.h>

#include <TMatrixD.h>

#include <TChain.h>
#include <TFile.h>
#include <TMinuit.h>
#include <TMinuitMinimizer.h>
#include <TSpectrum.h>

#include "TVirtualFitter.h"
#include "TFitter.h"
#include "TGraphErrors.h"
#include "TGraph2D.h"



/* ============================== */
/* Arrays with exp data */
#define MAXPOINTS 5000000
Double_t xP[MAXPOINTS];
Double_t yP[MAXPOINTS];
Double_t zP[MAXPOINTS];
Int_t nPoints = 0;
Int_t nPointsFront = 0;
Int_t nPointsBack = 0;

/* ============================== */
/* CAD data */
#define nSides 4
#define nCorners 4
#define nSurfaces 2
Double_t xCAD[nCorners][nSurfaces];
Double_t yCAD[nCorners][nSurfaces];
Double_t zCAD[nCorners][nSurfaces];
char *cSide[nSides];
char *cSurf[nSurfaces];


/*********************************************/

/* Distance of a point from the circle */
Double_t GetDistance(TVector3 Point, TVector3 Normal, Double_t Cost);


/*********************************************/
/* Histograms */
void makeHisto();
TObjArray Hlist(0);

TVector3 GetVector(Double_t theta, Double_t phi);
void PrintVector(char *name, TVector3 V);

/*********************************************/
void PrintUsage(char *processName);
using namespace std;

/*********************************************/
int main(Int_t argc, Char_t *argv[])
{
  char *file;
  char *argptr = NULL;

  char fname[200];
  char hname[200];
  char str[200];
  
  /* ======================================== */
  /* Scanning the command line */
  for (Int_t i=1; i<argc; i++) {
    argptr = argv[i];
    //cout << argv[i] << endl;
    
    if (*argptr == '-') {
      argptr++;
      switch (*argptr) {
	
      case 'F':
	file = (++argptr);
	//sprintf(file, "%s", ++argptr);
	break;
      case 'h':
	PrintUsage(argv[0]);
	break;
      default:
	fprintf(stderr,"Unrecognized argument: [-%s]\n\n", argptr);
	PrintUsage(argv[0]);
	break;
      }
      
    }
  }  


  /*********************/
  /* histograms */
  /*********************/
  makeHisto();





  FILE *fIn;
  FILE *fOut;
  TH1F *h1;


  /* ========================================== */
  /* Reading CAD  data */

  sprintf(fname, "%sCAD_corners.txt", file);
  fIn = fopen(&fname[0], "r");
  if (!fIn) {
    cout << "ERROR: No input file " << fname << endl;
    return -1;
  }

  for (Int_t s=0; s<nSurfaces; s++) {
    fgets(str, sizeof(str), fIn);
    cout << str;
    
    for (Int_t c=0; c<nCorners; c++) {

      fgets(str, sizeof(str), fIn);

      sscanf(str, "%lf %lf %lf\n", &xCAD[c][s], &yCAD[c][s], &zCAD[c][s]);
      cout << "  c=" << c << "  " << xCAD[c][s] << "  "  << yCAD[c][s] << "  "  << zCAD[c][s] << endl;
    }

  }
  
  fclose(fIn);





  /* =================================== */
  /* loop over the 4 sides of the mirror */
  Int_t corner1;
  Int_t corner2;
  Int_t Qcorner1;
  Int_t Qcorner2;
  Double_t Constant[nSides];
  TVector3 Normal[nSides];
  for (Int_t l=0; l<nSides; l++) {

    /* Choosing the mirror side */
    if (l == 0) {
      corner1 = 0;
      corner2 = 1;

      Qcorner1 = 2;
      Qcorner2 = 3;
    }
    else if (l == 1) {
      corner1 = 1;
      corner2 = 3;

      Qcorner1 = 0;
      Qcorner2 = 2;
    }
    else if (l == 2) {
      corner1 = 3;
      corner2 = 2;

      Qcorner1 = 1;
      Qcorner2 = 0;
    }
    else if (l == 3) {
      corner1 = 2;
      corner2 = 0;

      Qcorner1 = 3;
      Qcorner2 = 1;
    }

    cout << " ------------ SIDE: " << cSide[l] << endl;


    TVector3 P1(xCAD[corner1][0], yCAD[corner1][0], zCAD[corner1][0]);
    //PrintVector("P1", P1);
    TVector3 P2(xCAD[corner2][0], yCAD[corner2][0], zCAD[corner2][0]);
    //PrintVector("P2", P2);
    TVector3 P3(xCAD[corner2][1], yCAD[corner2][1], zCAD[corner2][1]);
    //PrintVector("P3", P3);
    TVector3 P4(xCAD[corner1][1], yCAD[corner1][1], zCAD[corner1][1]);
    //PrintVector("P4", P4);

    Normal[l] = (P2 - P1).Cross(P4 - P1).Unit();
    Constant[l] = Normal[l].Dot(P3);

    /* Check of the planarity of the 4 corners */
    Double_t q1 = (180./TMath::Pi()) * Normal[l].Angle( (P3 - P1).Unit());
    cout << "  q1=" << q1 << endl;

    /* Check that the 4 points are on the plane */
    Double_t c1 = Normal[l].Dot(P1);
    Double_t c2 = Normal[l].Dot(P2);
    Double_t c3 = Normal[l].Dot(P3);
    Double_t c4 = Normal[l].Dot(P4);
    cout << "  c=" << Constant[l] << "  c1=" << c1 << "  c2=" << c2 << "  c3=" << c3 << "  c4=" << c4 << endl;


    /* Checking that the normal is exiting from the mirror */
    /* Corners on the other side of the mirror */
    TVector3 Q1(xCAD[Qcorner1][0], yCAD[Qcorner1][0], zCAD[Qcorner1][0]);
    //PrintVector("Q1", Q1);
    TVector3 Q2(xCAD[Qcorner2][0], yCAD[Qcorner2][0], zCAD[Qcorner2][0]);
    //PrintVector("Q2", Q2);
    TVector3 Q3(xCAD[Qcorner2][1], yCAD[Qcorner2][1], zCAD[Qcorner2][1]);
    //PrintVector("Q3", Q3);
    TVector3 Q4(xCAD[Qcorner1][1], yCAD[Qcorner1][1], zCAD[Qcorner1][1]);
    //PrintVector("Q4", Q4);
    
    /* if direction>0 => the Normal is pointing inside the mirror 
       if direction<0 => the Normal is pointing outside the mirror */
    Double_t direction = Normal[l].Dot( (P1-Q1).Unit() );
    cout << "   direction=" << direction << endl;

  }
  cout << "==========================================" << endl;


  /* ======================================= */
  /* Printout of the CAD plane data */
  sprintf(fname, "%s.CADplanes.out", file);
  fOut = fopen(&fname[0], "w");
  for (Int_t l=0; l<nSides; l++) {
    fprintf(fOut, "%8.5f %8.5f %8.5f %12.5f\n", Normal[l].X(), Normal[l].Y(), Normal[l].Z(), Constant[l]);
  }
  fclose(fOut);





  /************************************************/
  /* Reading measured data */
  /************************************************/

  /* file for printout */
  sprintf(fname, "%s.calcP.out", file);
  fOut = fopen(&fname[0], "w");



  for (Int_t l=0; l<nSides; l++) {
  //for (Int_t l=0; l<1; l++) {
    nPoints = 0;
    nPointsBack = 0;
    nPointsFront = 0;
    TVector3 Point;
    Double_t Distance;

    /* ------------------------------------- */
    /* reading frontal surface */
    sprintf(fname, "%s_%s_front.txt", file, cSide[l]);
    fIn = fopen(&fname[0], "r");
    if (!fIn) {
      cout << "ERROR: No input file " << fname << endl;
      return -1;
    }
    else {
      cout << "  l=" << l << "   file: " << fname << endl;


      while (fgets(str, sizeof(str), fIn) ) {
	//cout << str;
	sscanf(str, "%lf %lf %lf\n", &xP[nPoints], &yP[nPoints], &zP[nPoints]);

	Point.SetXYZ(xP[nPoints], yP[nPoints], zP[nPoints]);
	Distance = GetDistance(Point, Normal[l], Constant[l]);


	sprintf(hname, "h_%s_%s", cSide[l], cSurf[0]);
	gDirectory->GetObject(hname, h1);
	h1->Fill(Distance);
	
	fprintf(fOut, "%d  %d  %12.3f %12.3f %12.3f %12.5f\n", l, 0, xP[nPoints], yP[nPoints], zP[nPoints], Distance);

	nPoints++;
      }

      fclose(fIn);
    }
    nPointsFront = nPoints;

    /* ------------------------------------- */
    /* reading back surface */
    sprintf(fname, "%s_%s_back.txt", file, cSide[l]);
    fIn = fopen(&fname[0], "r");
    if (!fIn) {
      cout << "ERROR: No input file " << fname << endl;
      return -1;
    }
    else {
      cout << "  l=" << l << "   file: " << fname << endl;

      while (fgets(str, sizeof(str), fIn) ) {
	//cout << str;
	sscanf(str, "%lf %lf %lf\n", &xP[nPoints], &yP[nPoints], &zP[nPoints]);

	Point.SetXYZ(xP[nPoints], yP[nPoints], zP[nPoints]);
	Distance = GetDistance(Point, Normal[l], Constant[l]);

	sprintf(hname, "h_%s_%s", cSide[l], cSurf[1]);
	gDirectory->GetObject(hname, h1);
	h1->Fill(Distance);

	fprintf(fOut, "%d  %d  %12.3f %12.3f %12.3f %12.5f\n", l, 1, xP[nPoints], yP[nPoints], zP[nPoints], Distance);
 
	nPoints++;
      }

      fclose(fIn);
    }
    nPointsBack = nPoints - nPointsFront;

  
    cout << "nPoints=" << nPoints << "  nPointsF=" << nPointsFront << "  nPointsB=" << nPointsBack << endl;

  }
  
  fclose(fOut);


  /*******************************/
  /* out histograms */
  sprintf(fname, "%s.calcP.root", file);
  TFile f(&fname[0], "recreate");
  //Hlist.ls();
  Hlist.Write();
  f.Close();

  return 1;
}
/* ===================================================== */
void makeHisto() 
{
  char name[200];
  char title[200];

  Int_t nbins = 100;
  Double_t xmax = 5.;

  for (Int_t s=0; s<nSurfaces; s++) {
    cSurf[s] = (char*)malloc(20);

    if (s == 0) {
      sprintf(cSurf[s], "front");
    }
    else if (s == 1) {
      sprintf(cSurf[s], "back");
    }


    for (Int_t l=0; l<nSides; l++) {
      cSide[l] = (char*)malloc(20);

      if (l == 0) {
	sprintf(cSide[l], "bottom");
      }
      else if (l == 1) {
	sprintf(cSide[l], "right");
      }
      else if (l == 2) {
	sprintf(cSide[l], "top");
      }
      else if (l == 3) {
	sprintf(cSide[l], "left");
      }
      
    
      sprintf(name, "h_%s_%s", cSide[l], cSurf[s]);
      sprintf(title, "#Delta R (mm), %s, %s", cSide[l], cSurf[s]);
      TH1F *hDR = new TH1F(name, title, nbins, -xmax, xmax);
      Hlist.Add(hDR);
    }
  }


  /* Distance of the measured point from the side */

  
}
/* ===================================================== */
Double_t GetDistance(TVector3 Point, TVector3 Normal, Double_t Cost)
{
  /* Calculate the distance of the Point from the plane
     Normal,Cost -> normal to the plane e constant term

  */
     

  Double_t d2plane = (Normal.Dot(Point) - Cost);

  return d2plane;
}
/* ===================================================== */
TVector3 GetVector(Double_t theta, Double_t phi)
{

  TVector3 V(TMath::Sin(theta) * TMath::Cos(phi), 
	     TMath::Sin(theta) * TMath::Sin(phi),
	     TMath::Cos(theta));

  return V;
}
/* ----------------------------------------------- */
void PrintVector(char *name, TVector3 V)
{

  fprintf(stdout, "%10s : X=%10.6f  Y=%10.6f  Z=%10.6f \n", name, V.X(), V.Y(), V.Z());

}
/* ===================================================== */
/* ===================================================== */
void PrintUsage(char *processName)
{
  fprintf(stderr,"Usage: %s [-Options] \n\n",processName);
  fprintf(stderr,"  Options:\n");

  fprintf(stderr,"\t-F[#]\t\tFile name\n");


  exit(0);
}

