



#define nSides 4
#define nCorners 4
#define nSurfaces 2
char *cSide[nSides];
char *cSurf[nSurfaces+1];

/* max tolerance on the dimensions from tech. spec. */
Double_t Tolerance = 0.25;
Double_t BigTolerance = 1.;


void PlotMirrorSides(char *mirror)
{

  char file[100];
  char hname[100];
  char title[100];

  /* Setting up name of the mirror sides */

  for (Int_t s=0; s<=nSurfaces; s++) {
    cSurf[s] = (char*)malloc(20);
    
    if (s == 0) {
      sprintf(cSurf[s], "front");
    }
    else if (s == 1) {
      sprintf(cSurf[s], "back");
    }
    else if (s == 2) {
      sprintf(cSurf[s], "all");
    }


    for (Int_t l=0; l<nSides; l++) {
      cSide[l] = (char*)malloc(20);

      if (l == 0) {
	sprintf(cSide[l], "bottom");
      }
      else if (l == 1) {
	sprintf(cSide[l], "right");
      }
      else if (l == 2) {
	sprintf(cSide[l], "top");
      }
      else if (l == 3) {
	sprintf(cSide[l], "left");
      }

    }

  }

  /*******************************************************/
  /* Opening root file */
  sprintf(file, "%s.calcP.root", mirror);
  TFile *fRoot = new TFile(file);
  if (fRoot->IsZombie()) {
    cout << "  error: No file " << file << endl;
    return;
  }
  cout << "Reading Root file: " << file << endl;

  /*******************************************************/
  /* file ps landscape */ 
  Char_t fname[100];
  sprintf(fname, "%s_calcP.ps", mirror);
  TPostScript *ps = new TPostScript(&fname[0], 112);


  /********************************************/
  /* Root canvas */
  TCanvas *c1 = new TCanvas("c1");
  ps->Range(26,20);

  /* ---------------------------------- */  
  /* per legende */
  Double_t sizeTXT = 0.04;
  Double_t dxLeg = 0.2;
  Double_t x1Leg, x2Leg;
  
  Double_t dyLeg = 0.04;
  Double_t y2Leg, y1Leg;

  Int_t color;
  Int_t marker;

  Double_t X[nSides];
  Double_t Y[nSides];

  /* ----------------------------------------- */
  /* Arrays with mean and RMS of the distances */
  Double_t meanR[nSides][nSurfaces+1];
  Double_t rmsR[nSides][nSurfaces+1];


  /* lines for dimension tolerance */
  TLine *Zero = new TLine(0.5, 0., 0.5+nSides, 0.);
  Zero->SetLineColor(1);
  Zero->SetLineStyle(2);

  TLine *lToleranceLow = new TLine(0.5, -Tolerance, 0.5+nSides, -Tolerance);
  lToleranceLow->SetLineColor(4);
  lToleranceLow->SetLineStyle(1);

  TLine *lToleranceUp = new TLine(0.5, Tolerance, 0.5+nSides, Tolerance);
  lToleranceUp->SetLineColor(4);
  lToleranceUp->SetLineStyle(1);

  TLine *lBigToleranceLow = new TLine(0.5, -BigTolerance, 0.5+nSides, -BigTolerance);
  lBigToleranceLow->SetLineColor(2);
  lBigToleranceLow->SetLineStyle(1);

  TLine *lBigToleranceUp = new TLine(0.5, BigTolerance, 0.5+nSides, BigTolerance);
  lBigToleranceUp->SetLineColor(2);
  lBigToleranceUp->SetLineStyle(1);


  /********************************************/
  /* loop over sides and surfaces */
  /********************************************/
  gStyle->SetOptStat(1111);

  for (Int_t l=0; l<nSides; l++) {

    TH1F *hS1;

    for (Int_t s=0; s<nSurfaces; s++) {
      sprintf(hname, "h_%s_%s", cSide[l], cSurf[s]);
      TH1F *h1 = (TH1F*)fRoot->Get(hname);

      color = 2 * (s + 1);
      h1->SetLineColor(color);
      h1->Draw();
      c1->Update();

      meanR[l][s] = h1->GetMean();
      rmsR[l][s] = h1->GetRMS();

      if (s == 0) {
	sprintf(hname, "h_%s", cSide[l]);
	hS1 = (TH1F*)h1->Clone(hname);
	sprintf(title, "#Delta R (mm), %s", cSide[l]);
	hS1->SetTitle(title);
      }
      else {
	hS1->Add(h1);
      }
    }

    color = 1;
    hS1->SetLineColor(color);
    hS1->Draw();
    c1->Update();

    meanR[l][nSurfaces] = hS1->GetMean();
    rmsR[l][nSurfaces] = hS1->GetRMS();
  }



  /*******************************************************/
  /* plots of means */
  /*******************************************************/

  x2Leg = 0.9;
  x1Leg = x2Leg - dxLeg;
  y2Leg = 0.9;
  y1Leg = y2Leg - dyLeg;


  gStyle->SetOptStat(0);


  Double_t ymin = -5;
  Double_t ymax = 5;
  TH2F *h2 = new TH2F("h2", "", 2, 0.5, nSides+0.5, 2, ymin, ymax);
  h2->GetXaxis()->SetTitle("Mirror Side ID");
  h2->GetYaxis()->SetTitle("Mean Deviation (mm)");
  h2->Draw();

  Zero->Draw("same");

  lToleranceLow->Draw("same");
  lToleranceUp->Draw("same");

  lBigToleranceLow->Draw("same");
  lBigToleranceUp->Draw("same");


  for (Int_t s=0; s<nSurfaces; s++) {


    for (Int_t l=0; l<nSides; l++) {
      X[l] = l + 1;
      Y[l] = meanR[l][s];
    }

    marker = 24+s;
    color = (s+1) * 2;
    color = 1;
    if (s == 2) {
      marker = 20;
      color = 1;
    }

    TGraph *gMean = new TGraph(nSides, X, Y);
    gMean->SetMarkerStyle(marker);
    gMean->SetMarkerColor(color);
    gMean->SetLineColor(color);
    gMean->SetMarkerSize(1.1);
    gMean->Draw("P");

    TLegend *lMean = new TLegend(x1Leg, y1Leg, x2Leg, y2Leg);
    lMean->SetTextColor(color);
    lMean->SetTextSize(sizeTXT);
    lMean->AddEntry(gMean, cSurf[s], "P");
    lMean->Draw();
    y2Leg = y2Leg - dyLeg;
    y1Leg = y1Leg - dyLeg;

  }
  
  c1->Update();



  /*******************************************************/
  /* plots of rms */
  /*******************************************************/

  x2Leg = 0.9;
  x1Leg = x2Leg - dxLeg;
  y2Leg = 0.9;
  y1Leg = y2Leg - dyLeg;


  gStyle->SetOptStat(0);


  Double_t ymin = 0;
  Double_t ymax = 3;
  TH2F *h2 = new TH2F("h2", "", 2, 0.5, nSides+0.5, 2, ymin, ymax);
  h2->GetXaxis()->SetTitle("Mirror Side ID");
  h2->GetYaxis()->SetTitle("RMS Deviation (mm)");
  h2->Draw();

  //Zero->Draw("same");

  //lToleranceLow->Draw("same");
  lToleranceUp->Draw("same");

  //lBigToleranceLow->Draw("same");
  lBigToleranceUp->Draw("same");
  

  for (Int_t s=0; s<nSurfaces; s++) {


    for (Int_t l=0; l<nSides; l++) {
      X[l] = l + 1;
      Y[l] = rmsR[l][s];
    }

    marker = 24+s;
    //color = (s+1) * 2;
    color = 1;
    if (s == 2) {
      marker = 20;
      color = 1;
    }

    TGraph *gRms = new TGraph(nSides, X, Y);
    gRms->SetMarkerStyle(marker);
    gRms->SetMarkerColor(color);
    gRms->SetLineColor(color);
    gRms->SetMarkerSize(1.1);
    gRms->Draw("P");

    TLegend *lRms = new TLegend(x1Leg, y1Leg, x2Leg, y2Leg);
    lRms->SetTextColor(color);
    lRms->SetTextSize(sizeTXT);
    lRms->AddEntry(gRms, cSurf[s], "P");
    lRms->Draw();
    y2Leg = y2Leg - dyLeg;
    y1Leg = y1Leg - dyLeg;

  }
  
  c1->Update();


  /*******************************************************/
  ps->Close();
  c1->Close();
  fRoot->Close();
}
